library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_textio.all;
use std.textio.all;
use std.env.stop;

entity write_stimuli_tb is
end write_stimuli_tb;

architecture rtl of write_stimuli_tb is
--*************************************************************************************
--*************************************************************************************
	component fsm_stimuli is
		port(
			signal clk	 : in std_logic;
			signal tc_write : in std_logic;
			signal text_write : in std_logic;
			signal write_tc : in string(1 to 32);
			signal reset : in std_logic;
			signal mysimena	: in std_logic;
			signal mypreset : in std_logic;
			signal myreset	: in std_logic;
			signal myenable : in std_logic;
			signal myclk	: in std_logic;
			signal myd		: in std_logic;
			signal myq		: in std_logic
		);
	end component;
--*************************************************************************************
--*************************************************************************************
	signal simclk 	: std_logic;
	signal simtc_write	: std_logic;
	signal simtext_write : std_logic;
	signal simreset : std_logic;
	signal simwrite_tc	: string(1 to 32);
	signal simsimena	: std_logic;
	signal simpreset : std_logic;
	signal sim_reset	: std_logic;
	signal simenable : std_logic;
	signal sim_clk	: std_logic;
	signal simd		: std_logic;
	signal simq		: std_logic;
	

	
--*************************************************************************************
-- funzione string pad
--*************************************************************************************
	function padded_string(s: string; n: positive) return string is
		variable ps: string(1 to n) := (others => ' ');
			begin
			if s'length >= n then
				ps := s(1 to n); --- tronca la stringa
			else
				ps(1 to s'length) := s;
				ps(s'length + 1 to n) := (others => ' ');
			end if;
		return ps;
	end;

	
	begin
--*************************************************************************************
--*************************************************************************************
		uut: fsm_stimuli port map(
			clk => simclk,
			reset => simreset,
			tc_write => simtc_write,
			text_write => simtext_write,
			write_tc => simwrite_tc,
			mysimena => simsimena,
			mypreset => simpreset,
			myreset => sim_reset,
			myenable => simenable,
			myclk => sim_clk,
			myd => simd,
			myq => simq
		);
--*************************************************************************************
-- processi dei segnali
--
-- in questa sezione bisogna descvire i process dei segnali del componente
-- sotto test.
--*************************************************************************************
		simena_process: process
		begin
			simsimena <= '0';
			wait for 10 ns;
			simsimena <= '1';
			wait for 70 ns;
			simsimena <= '0';
			wait for 10 ns;
			simsimena <= '1';
			wait;
	end process;
	
	
	d_process: process
		begin	
			simd <= '0';
			wait for 40 ns;
			simd <= '1';
			wait for 80 ns;
			simd <= '0';
			wait for 20 ns;
			simd <= '1';
			wait for 20 ns;
			simd <= '0';
			wait;
	end process;
			
		
	myreset_process: process
		begin
			sim_reset <= '1';
			wait for 10 ns;
			sim_reset <= '0';
			wait for 30 ns;
			sim_reset <= '1';
			wait for 10 ns;
			sim_reset <= '0';
			wait for 30 ns;
			sim_reset <= '1';
			wait for 20 ns;
			sim_reset <= '0';
			wait for 100 ns;
			sim_reset <= '1';
			wait for 10 ns;
			sim_reset <= '0';
			wait for 10 ns;
			sim_reset <= '1';
			wait;
	end process;
			

	mypreset_process: process
		begin
			simpreset <= '0';
			wait for 20 ns;
			simpreset <= '1';
			wait for 10 ns;
			simpreset <= '0';
			wait for 30 ns;
			simpreset <= '1';
			wait for 10 ns;
			simpreset <= '0';
			wait for 140 ns;
			simpreset <='1';
			wait;
	end process;
			

	myenable_process: process
		begin
			simenable <= '0';
			wait for 80 ns;
			simenable <= '1';
			wait for 80 ns;
			simenable <= '0';
			wait for 30 ns;
			simenable <= '1';
			wait;
	end process;
			
	
	myclk_process: process
		begin
			sim_clk <= '0';
			wait for 10 ns;
			sim_clk <= '1';
			wait for 10 ns;
	end process;
			
			
	myq_process: process
		begin
			simq <= '0';
			wait for 20 ns;
			simq <= '1';
			wait for 20 ns;
			simq <= '0';
			wait for 20 ns;
			simq <= '1';
			wait for 20 ns;
			simq <= '0';
			wait for 30 ns;
			simq <= '1';
			wait for 20 ns;
			simq <= '0';
			wait for 20 ns;
			simq <= '1';
			wait for 50 ns;
			simq <= '0';
			wait for 10 ns;
			simq <= '1';
			wait for 10 ns;
			simq <= '0';
			wait;
	end process;
--*************************************************************************************
-- clock di simulazione	
--*************************************************************************************
	simclk_process: process
		begin
		simclk <= '0';
		wait for 10 ns;
		simclk <= '1';
		wait for 10 ns;
	end process;
--*************************************************************************************
-- simreset
--*************************************************************************************
	simreset_process: process
		begin
			simreset <= '1';
			wait for 5 ns;
			simreset <= '0';
			simtext_write <= '1';
			wait for 5 ns;
			simtext_write <= '0';
			wait;
	end process;
--*************************************************************************************
-- write test case
--*************************************************************************************
write_tc_process: process
		begin
			simtc_write <= '0';
			wait for 20 ns;
			simtc_write <= '1';
			simwrite_tc <= padded_string("#tc-1", 32);									
			wait for 10 ns;
			simtc_write <= '0';
			wait for 60 ns;
			simtc_write <= '1';
			simwrite_tc <= padded_string("#tc-2", 32);
			wait for 10 ns;
			simtc_write <= '0';
			wait for 20 ns;
			simtc_write <= '1';
			simwrite_tc <= padded_string("#tc-3", 32);
			wait for 10 ns;
			simtc_write <= '0';
			wait for 40 ns;
			simtc_write <= '1';
			simwrite_tc <= padded_string("#tc-4", 32);
			wait for 10 ns;
			simtc_write <= '0';
			wait;
	end process;
	
end rtl;